94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
171 lines
5.6 KiB
Plaintext
171 lines
5.6 KiB
Plaintext
/*
|
|
* Copyright (c) 2024 by FlashInfer team.
|
|
*
|
|
* Licensed under the Apache License, Version 2.0 (the "License");
|
|
* you may not use this file except in compliance with the License.
|
|
* You may obtain a copy of the License at
|
|
*
|
|
* http://www.apache.org/licenses/LICENSE-2.0
|
|
*
|
|
* Unless required by applicable law or agreed to in writing, software
|
|
* distributed under the License is distributed on an "AS IS" BASIS,
|
|
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
|
|
* See the License for the specific language governing permissions and
|
|
* limitations under the License.
|
|
*/
|
|
|
|
#include <ATen/cuda/CUDAContext.h>
|
|
#include <c10/cuda/CUDAGuard.h>
|
|
#include <torch/all.h>
|
|
|
|
#ifndef USE_ROCM
|
|
|
|
#include <flashinfer/activation.cuh>
|
|
|
|
#include "utils.h"
|
|
|
|
#else
|
|
#include "hip/hip_act_and_mul.cuh"
|
|
#endif
|
|
|
|
// Adapted from flashinfer activation
|
|
// https://github.com/flashinfer-ai/flashinfer/blob/4e8eb1879f9c3ba6d75511e5893183bf8f289a62/csrc/activation.cu#L44
|
|
|
|
namespace detail {
|
|
|
|
template <typename T>
|
|
__device__ __forceinline__ float to_f32(const T& x) {
|
|
#if USE_ROCM
|
|
return castToFloat(x);
|
|
#else
|
|
return static_cast<float>(x);
|
|
#endif
|
|
}
|
|
|
|
template <typename T>
|
|
__device__ __forceinline__ T from_f32(float f32) {
|
|
#if USE_ROCM
|
|
return castFromFloat<T>(f32);
|
|
#else
|
|
return static_cast<T>(f32);
|
|
#endif
|
|
}
|
|
|
|
} // namespace detail
|
|
|
|
template <typename T>
|
|
__device__ __forceinline__ T silu(const T& x) {
|
|
float f32_val = detail::to_f32(x);
|
|
return detail::from_f32<T>(f32_val / (1.0f + expf(-f32_val)));
|
|
}
|
|
|
|
template <typename T>
|
|
__device__ __forceinline__ T gelu(const T& x) {
|
|
constexpr float kAlpha = M_SQRT1_2;
|
|
float f32_val = detail::to_f32(x);
|
|
return detail::from_f32<T>(f32_val * (0.5f * (1.0f + erf(f32_val * kAlpha))));
|
|
}
|
|
|
|
// gelu_quick(x) = x * torch.sigmoid(1.702 * x)
|
|
template <typename T>
|
|
__device__ __forceinline__ T gelu_quick_act(const T& x) {
|
|
float f32_val = detail::to_f32(x);
|
|
return detail::from_f32<T>(f32_val / (1.0f + expf(-f32_val * 1.702f)));
|
|
}
|
|
|
|
template <typename T>
|
|
__device__ __forceinline__ T gelu_tanh(const T& x) {
|
|
constexpr float kAlpha = 0.044715f;
|
|
constexpr float kBeta = 0.7978845608028654f;
|
|
float f32_val = detail::to_f32(x);
|
|
const float cdf = 0.5f * (1.0f + tanhf((kBeta * (f32_val + kAlpha * f32_val * f32_val * f32_val))));
|
|
return detail::from_f32<T>(f32_val * cdf);
|
|
}
|
|
|
|
void silu_and_mul(at::Tensor& out, at::Tensor& input) {
|
|
int d = input.size(-1) / 2;
|
|
int64_t num_tokens = input.numel() / input.size(-1);
|
|
dim3 grid(num_tokens);
|
|
|
|
const cudaStream_t stream = at::cuda::getCurrentCUDAStream();
|
|
const at::cuda::OptionalCUDAGuard device_guard(device_of(input));
|
|
|
|
DISPATCH_PYTORCH_DTYPE_TO_CTYPE_FLOAT_FP16(input.scalar_type(), c_type, [&] {
|
|
uint32_t vec_size = 16 / sizeof(c_type);
|
|
dim3 block(std::min(d / vec_size, 1024U));
|
|
#if USE_ROCM
|
|
sgl_hip::activation::act_and_mul_kernel<c_type, silu>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#else
|
|
flashinfer::activation::act_and_mul_kernel<c_type, silu>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#endif
|
|
return true;
|
|
});
|
|
}
|
|
|
|
void gelu_tanh_and_mul(at::Tensor& out, at::Tensor& input) {
|
|
int d = input.size(-1) / 2;
|
|
int64_t num_tokens = input.numel() / input.size(-1);
|
|
dim3 grid(num_tokens);
|
|
|
|
const cudaStream_t stream = at::cuda::getCurrentCUDAStream();
|
|
const at::cuda::OptionalCUDAGuard device_guard(device_of(input));
|
|
|
|
DISPATCH_PYTORCH_DTYPE_TO_CTYPE_FLOAT_FP16(input.scalar_type(), c_type, [&] {
|
|
uint32_t vec_size = 16 / sizeof(c_type);
|
|
dim3 block(std::min(d / vec_size, 1024U));
|
|
#if USE_ROCM
|
|
sgl_hip::activation::act_and_mul_kernel<c_type, gelu_tanh>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#else
|
|
flashinfer::activation::act_and_mul_kernel<c_type, gelu_tanh>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#endif
|
|
return true;
|
|
});
|
|
}
|
|
|
|
void gelu_and_mul(at::Tensor& out, at::Tensor& input) {
|
|
int d = input.size(-1) / 2;
|
|
int64_t num_tokens = input.numel() / input.size(-1);
|
|
dim3 grid(num_tokens);
|
|
|
|
const cudaStream_t stream = at::cuda::getCurrentCUDAStream();
|
|
const at::cuda::OptionalCUDAGuard device_guard(device_of(input));
|
|
|
|
DISPATCH_PYTORCH_DTYPE_TO_CTYPE_FLOAT_FP16(input.scalar_type(), c_type, [&] {
|
|
uint32_t vec_size = 16 / sizeof(c_type);
|
|
dim3 block(std::min(d / vec_size, 1024U));
|
|
#if USE_ROCM
|
|
sgl_hip::activation::act_and_mul_kernel<c_type, gelu>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#else
|
|
flashinfer::activation::act_and_mul_kernel<c_type, gelu>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
#endif
|
|
|
|
return true;
|
|
});
|
|
}
|
|
|
|
#if USE_ROCM
|
|
void gelu_quick(at::Tensor& out, const at::Tensor& input) {
|
|
int d = input.size(-1);
|
|
int64_t num_tokens = input.numel() / input.size(-1);
|
|
dim3 grid(num_tokens);
|
|
|
|
const cudaStream_t stream = at::cuda::getCurrentCUDAStream();
|
|
const at::cuda::OptionalCUDAGuard device_guard(device_of(input));
|
|
|
|
DISPATCH_PYTORCH_DTYPE_TO_CTYPE_FLOAT_FP16(input.scalar_type(), c_type, [&] {
|
|
uint32_t vec_size = 16 / sizeof(c_type);
|
|
dim3 block(std::min(d / vec_size, 1024U));
|
|
sgl_hip::activation::act_only_kernel<c_type, gelu_quick_act>
|
|
<<<grid, block, 0, stream>>>(static_cast<c_type*>(out.data_ptr()), static_cast<c_type*>(input.data_ptr()), d);
|
|
|
|
return true;
|
|
});
|
|
}
|
|
#endif
|